--- Trigger 1 --- Anlegen Segelschiffen
CREATE OR REPLACE FUNCTION fSegelschiffInsert() RETURNS TRIGGER AS $$
DECLARE
   v_sid INTEGER;
BEGIN
   IF NEW.sid IS NULL THEN
       SELECT COALESCE(MAX(sid),0)+1 INTO v_sid FROM Segelschiff s WHERE s.hafen = NEW.hafen;
       NEW.sid := v_sid;
   END IF;
   RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER trSegelschiffInsert BEFORE INSERT
   ON Segelschiff FOR EACH ROW EXECUTE PROCEDURE fSegelschiffInsert();

--- Trigger 2 --- Schiff kann nur gechartert werden falls verfuegbar und genug Kapazitaet
CREATE OR REPLACE FUNCTION fGechartert() RETURNS TRIGGER AS $$
BEGIN
   IF EXISTS(SELECT * 
             FROM Gechartert g 
              JOIN Tour t ON g.tid = t.tid 
             WHERE (g.startdatum, g.startdatum + t.dauer) OVERLAPS (NEW.startdatum,NEW.startdatum + (SELECT dauer FROM Tour WHERE tid=NEW.tid)) AND
                   g.hafen = NEW.hafen AND g.sid = NEW.sid) THEN
      RAISE EXCEPTION 'Segelschiff in dieser Zeit bereits gechartert!';
   END IF;
   
   IF EXISTS(SELECT * 
             FROM Segelschiff
             WHERE hafen=NEW.hafen AND sid = NEW.sid AND
                   kapazitaet < (SELECT COUNT(*) FROM CrewMitglieder WHERE cleiter = NEW.cleiter AND cid = NEW.cid)) THEN
      RAISE EXCEPTION 'Segelschiff hat nicht genug Kapazitaet!';
   END IF;
   
   RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER trGechartert BEFORE INSERT
   ON Gechartert FOR EACH ROW EXECUTE PROCEDURE fGechartert();
   
--- Trigger 3 --- Tourteile
CREATE OR REPLACE FUNCTION fTeilTour() RETURNS TRIGGER AS $$
BEGIN
   IF TG_OP = 'INSERT' OR TG_OP='UPDATE' THEN
      UPDATE Tour 
      SET dauer = dauer + (SELECT dauer FROM Tour WHERE tid = NEW.tid)
      WHERE tid = NEW.teilvon;
   END IF;
   
   IF TG_OP = 'UPDATE' OR TG_OP='DELETE' THEN
      UPDATE Tour
      SET dauer = dauer - (SELECT dauer FROM Tour WHERE tid = OLD.tid)
      WHERE tid = OLD.teilvon;
   END IF;
   
   RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER trTeilTour AFTER INSERT OR UPDATE OR DELETE
   ON TeilTour FOR EACH ROW EXECUTE PROCEDURE fTeilTour();


CREATE OR REPLACE FUNCTION fTourDauer() RETURNS TRIGGER AS $$
BEGIN
   IF NEW.dauer <> OLD.dauer THEN
      UPDATE Tour
      SET dauer = dauer - OLD.dauer
      WHERE tid IN (SELECT teilvon FROM TeilTour WHERE tid = OLD.tid);
      
      UPDATE Tour
      SET dauer = dauer + NEW.dauer
      WHERE tid IN (SELECT teilvon FROM TeilTour WHERE tid = NEW.tid);
   END IF;
   
   RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER trTourDauer AFTER UPDATE
   ON Tour FOR EACH ROW EXECUTE PROCEDURE fTourDauer();
   

--- Procedure 1 --- Anlegen einer Crew
CREATE OR REPLACE FUNCTION CreateCrew(p_skid INTEGER, p_name VARCHAR(20), p_cnt_ersthelfer INTEGER, p_cnt_koeche INTEGER, p_cnt_mitglieder INTEGER) RETURNS void AS $$
DECLARE
    v_cid INTEGER;
    rec RECORD;
BEGIN
    IF p_cnt_ersthelfer  < 2 THEN
       RAISE EXCEPTION 'Mindestens 2 Ersthelfer!';
    END IF;
    
    IF p_cnt_koeche < 1 OR p_cnt_koeche > 4 THEN
       RAISE EXCEPTION 'Mindestens 1 und nicht mehr als 4 Koeche!';
    END IF;
    
    --Finde eine cid fuer den Skipper mit skid
    SELECT COALESCE(MAX(cid),0)+1 INTO v_cid FROM Crew c WHERE c.leiter = p_skid;
    
    INSERT INTO Crew VALUES (p_skid,v_cid,p_name);
    INSERT INTO CrewMitglieder VALUES (p_skid,p_skid,v_cid);
    
    -- Fuege Koeche hinzu die noch nicht Teil der Crew sind
    FOR rec IN SELECT id, COUNT(koch) AS cnt 
               FROM koch LEFT JOIN CrewKoch ON id = koch 
               WHERE id NOT IN (SELECT segler FROM CrewMitglieder WHERE cleiter = p_skid AND cid = v_cid)
               GROUP BY id 
               ORDER BY cnt 
               LIMIT p_cnt_koeche LOOP
       INSERT INTO CrewMitglieder VALUES (rec.id,p_skid,v_cid);
       INSERT INTO CrewKoch VALUES (rec.id,p_skid,v_cid);
    END LOOP;
    
    -- Fuege Ersthelfer hinzu die noch nicht Teil der Crew sind
    FOR rec IN SELECT id, COUNT(ersthelfer) AS cnt 
               FROM ersthelfer LEFT JOIN CrewErsthelfer ON id = ersthelfer 
               WHERE id NOT IN (SELECT segler FROM CrewMitglieder WHERE cleiter = p_skid AND cid = v_cid)
               GROUP BY id 
               ORDER BY cnt 
               LIMIT p_cnt_ersthelfer LOOP
       INSERT INTO CrewMitglieder VALUES (rec.id,p_skid,v_cid);
       INSERT INTO CrewErsthelfer VALUES (rec.id,p_skid,v_cid);
    END LOOP;
    
    -- Fuege CrewMitglieder hinzu die noch nicht Teil der Crew sind
    FOR rec IN SELECT id, COUNT(segler) AS cnt 
               FROM Segler LEFT JOIN CrewMitglieder ON id = segler 
               WHERE id NOT IN (SELECT segler FROM CrewMitglieder WHERE cleiter = p_skid AND cid = v_cid)
               GROUP BY id 
               ORDER BY cnt 
               LIMIT p_cnt_mitglieder LOOP
       INSERT INTO CrewMitglieder VALUES (rec.id,p_skid,v_cid);
    END LOOP;
    
    RAISE NOTICE 'Crewid: %', v_cid;
END;
$$ LANGUAGE plpgsql;

